// TodayWindow.h: interface for the CTodayWindow class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(TODAYWINDOW_H_)
#define TODAYWINDOW_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <windows.h>
#include <aygshell.h>
#include <todaycmn.h>

class CTodayWindow  
{
public:
	// Member Variables
	HWND m_hWnd;

	// Construction/Destruction
	CTodayWindow();
	virtual ~CTodayWindow();

	// Main Create method
	BOOL Create(TODAYLISTITEM *ptli, HWND hWndParent, DWORD dwStyle = WS_VISIBLE | WS_CHILD);

	// Update Window
	void RefreshWindow(BOOL bShow = FALSE);

	// Set Methods
	BOOL SetIcon(UINT uID, int xDrawAt = 2, int yDrawAt = 0);
	void SetItemHeight(int nHeight);
	void SetClassInfo(LPCTSTR lpszClassName, LPCTSTR lpszWindowName);
	void SetInstance(HINSTANCE hInstance);

	// Get Methods
	HWND GetParent() {return m_hParentWnd;};
	int GetItemHeight() {return m_nHeight;};
	HINSTANCE GetInstance() {return m_hInstance;};
	HICON GetIcon() {return m_hIcon;};
	LPCTSTR GetClassName() {return m_lpszClassName;};
	LPCTSTR GetWindowName() {return m_lpszWindowName;};

	// Register/Unregister TodayWindow
	void RegisterTodayClass(WNDPROC wndProc);
	void UnregisterTodayClass();

	// TodayWndProc - main message loop
	virtual LRESULT CALLBACK TodayWndProc(UINT uMsg, WPARAM wParam, LPARAM lParam);
protected:
	int m_nHeight;
	POINT m_pointIconPos;

	HWND m_hParentWnd;
	HICON m_hIcon;
	HINSTANCE m_hInstance;

	LPCTSTR m_lpszClassName;
	LPCTSTR m_lpszWindowName;

	COLORREF m_crTodayText;
	HFONT m_fontTodayNormal;
	HFONT m_fontTodayBold;

	virtual void DrawTodayIcon(HDC hDC);
	virtual void GetTodayDefaults();

	// Message handlers
	virtual int OnCreate(LPCREATESTRUCT lpCreateStruct);
	virtual void OnDestroy();
	virtual void OnPaint(HDC hDC);
	virtual void OnEraseBkgnd(HDC hDC);
	virtual void OnTodayCustomQueryRefreshCache(TODAYLISTITEM *pTodayListItem, BOOL *pResult);
	virtual BOOL OnTodayCustomClearCache(TODAYLISTITEM *pTodayListItem);
	virtual void OnLButtonUp(UINT nFlags, POINT point);
	virtual void OnLButtonDown(UINT nFlags, POINT point);
	virtual void OnSettingChange(UINT nFlags, LPCTSTR lpszSection);
	virtual LRESULT OnNotify(UINT nID, NMHDR* pNMHDR);
	virtual LRESULT OnMessage(UINT uMsg, WPARAM wParam, LPARAM lParam);
};

#endif
